from SignerPy import *
def host() -> list or []:
    hosts = [
    "api16-normal-no1a.tiktokv.eu",
    "api16-normal-c-alisg.tiktokv.com",
    "api19-normal-c-alisg.tiktokv.com",
    "api16-normal-c-useast2a.tiktokv.com",
    "api16-normal-useast5.tiktokv.us",
    "api16-core-aion-useast5.us.tiktokv.com",
    "api16-normal-aion-useast5.us.tiktokv.com",
    "api16-normal-apix-quic.tiktokv.com",
    "api16-normal-apix.tiktokv.com",
    "api16-normal-baseline.tiktokv.com",
    "api16-normal-c-useast1a.tiktokv.com",
    "api16-normal-c-useast1a.musical.ly",
    "api16-normal-quic.tiktokv.com",
    "api16-normal-useast5.us.tiktokv.com",
    "api16-normal-useast8.us.tiktokv.com",
    "api16-normal-va.tiktokv.com",
    "api16-normal-vpc2-useast5.us.tiktokv.com",
    "api16-normal-zr.tiktokv.com",
    "api16-normal.tiktokv.com",
    "api16-normal.ttapis.com",
    "api19-core-c-alisg.tiktokv.com",
    "api19-core-c-useast1a.tiktokv.com",
    "api19-core-useast5.us.tiktokv.com",
    "api19-core-va.tiktokv.com",
    "api19-core-zr.tiktokv.com",
    "api19-core.tiktokv.com",
    "api19-normal-c-useast1a.musical.ly",
    "api19-normal-c-useast1a.tiktokv.com",
    "api19-normal-useast5.us.tiktokv.com",
    "api19-normal-va.tiktokv.com",
    "api19-normal-zr.tiktokv.com",
    "api19-normal.tiktokv.com",
    "api2-19-h2.musical.ly",
    "api2.musical.ly",
    "api21-core-c-alisg.tiktokv.com",
    "api21-core-va.tiktokv.com",
    "api21-core.tiktokv.com",
    "api21-h2-eagle.tiktokv.com",
    "api21-h2.tiktokv.com",
    "api21-normal.tiktokv.com",
    "api21-va.tiktokv.com",
    "api22-core-c-alisg.tiktokv.com",
    "api22-core-c-useast1a.tiktokv.com",
    "api22-core-va.tiktokv.com",
    "api22-core-zr.tiktokv.com",
    "api22-core.tiktokv.com",
    "api22-h2-eagle.tiktokv.com",
    "api22-normal-c-alisg.tiktokv.com",
    "api22-normal-c-useast1a.tiktokv.com"
]
    return hosts